/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include <stdio.h>
#include "platform.h"  //Contains Cache initializing function
#include "xparameters.h"  //The hardware configuration describing constants
#include "xgpio.h"	//GPIO API functions
#include "xintc.h"	//Interrupt Controller API functions
#include "xil_io.h"	//Contains the Xil_Out32 and Xil_In32 functions
#include "mb_interface.h" //Contains the functions for registering the
						  //interrupt controller with the microblaze MP
#include "ac97_demo.h"
#include "ac97.h"

#include <math.h>
/* ------------------------------------------------------------ */
/*				XPAR Constants									*/
/* ------------------------------------------------------------ */
/* All constants used from xparameters.h are located here so 	*/
/* that they may be easily accessed.							*/
/**/

#define AC97_BASEADDR XPAR_DIGILENT_AC97_CNTLR_BASEADDR
#define BTNS_BASEADDR XPAR_PUSH_BUTTONS_5BITS_BASEADDR
#define SW_BASEADDR XPAR_DIP_SWITCHES_8BITS_BASEADDR
#define LED_BASEADDR XPAR_LEDS_8BITS_BASEADDR
#define BTNS_DEVICE_ID XPAR_PUSH_BUTTONS_5BITS_DEVICE_ID
#define INTC_DEVICE_ID XPAR_INTC_0_DEVICE_ID
#define BTNS_IRPT_ID XPAR_INTC_SINGLE_DEVICE_ID
#define DDR2_BASEADDR XPAR_MCB_DDR2_S0_AXI_BASEADDR

/*
 * The pAudioMem constant points to the physical address of the
 * start of the audio data in the DDR2 memory. It is offset from
 * the beginning of DDR to allow room for program memory
 */
#define pAudioData (DDR2_BASEADDR + 0x01000000)

/* ------------------------------------------------------------ */
/*				Global Variables								*/
/* ------------------------------------------------------------ */

volatile u32 lBtnStateOld;
volatile u8 fsRunAction; //flags are set by interrupts to trigger
						 //different actions


/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */

int main()
{
    init_platform();

    //IO
    long int Sample_AC97_in  = 0;
    long int fft_re;
    long int  fft_im;
    long int  ifft_re;
    long int  ifft_im;
    long int Sample_AC97_out = 0;

    long int fft_re_in[64];
    long int fft_im_in[64];
    long int  ifft_re_out[64];
    long int  ifft_im_out[64];



    //Flag
    int sample_count = 0; //1-32
    int flg_first_64_sample = 0; //first 64 sample pass = 2
    int i= 0;

	/*
	 * Wait for AC97 to become ready
	 */
    while (!(AC97_Link_Is_Ready (AC97_BASEADDR)));

    /*
     * Set TAG to configure codec
     */
    AC97_Set_Tag_And_Id (AC97_BASEADDR, 0xF800);

    /*
     * Enable audio output and set volume
     */
    AC97_Unmute (AC97_BASEADDR, AC97_MASTER_VOLUME_OFFSET);
    AC97_Unmute (AC97_BASEADDR, AC97_HEADPHONE_VOLUME_OFFSET);
    AC97_Unmute (AC97_BASEADDR, AC97_PCM_OUT_VOLUME_OFFSET);

    AC97_Set_Volume (AC97_BASEADDR, AC97_MASTER_VOLUME_OFFSET,
       						     BOTH_CHANNELS, VOLUME_MID);
    AC97_Set_Volume (AC97_BASEADDR, AC97_HEADPHONE_VOLUME_OFFSET,
       						     BOTH_CHANNELS, VOLUME_MID);
    AC97_Set_Volume (AC97_BASEADDR, AC97_PCM_OUT_VOLUME_OFFSET,
       						     BOTH_CHANNELS, VOLUME_MID);
	//MIC Mute
	AC97_Mute (AC97_BASEADDR, AC97_MIC_VOLUME_OFFSET);

    while (1)
    {

    	if(Xil_In32(SW_BASEADDR) & bitSw0){

    		AC97_Set_Tag_And_Id (AC97_BASEADDR, 0xF800); //Set to configure
    		AC97_Mute (AC97_BASEADDR, AC97_MIC_VOLUME_OFFSET);

			//set record gain
			AC97_Unmute (AC97_BASEADDR, AC97_RECORD_GAIN_OFFSET);
			AC97_Set_Volume (AC97_BASEADDR, AC97_RECORD_GAIN_OFFSET,BOTH_CHANNELS, GAIN_MID);
			AC97_Set_Tag_And_Id (AC97_BASEADDR, 0x9800); //Set to Send/Receive data



			Xil_Out32(LED_BASEADDR, bitRecLED); //turn on LED
			//PLAY LOOP
			while(Xil_In32(SW_BASEADDR) & bitSw0){
                    sample_count += 1;

					// Block execution until next frame is ready
                    AC97_Wait_For_New_Frame (AC97_BASEADDR);

					//input sample
					Sample_AC97_in = XIo_In32(AC97_BASEADDR + AC97_PCM_IN_L_OFFSET);
					Sample_AC97_in = Sample_AC97_in >> 2;


					//output win - fft operation Circuit
					XIo_Out32 ((XPAR_WIN_OUT_BASEADDR), Sample_AC97_in);


					/*
					 * usually cycle
					 */
					if(flg_first_64_sample == 64){


						//FFT to CPU (real)
						if(sample_count == 1){
							for(i = 0; i < 64 ; i++){
								 fft_re= XIo_In32(XPAR_FFT_RE_IN_BASEADDR);
								 fft_re_in[i] = fft_re;
							}

						}

						//FFT to CPU (image)
						else if(sample_count == 2){
							for(i = 0; i <64; i++){
								fft_im = XIo_In32(XPAR_FFT_IM_IN_BASEADDR);
								fft_im_in[i] = fft_im;
							}
						}


						/*
						 * Swap real and image data
						 */
						else if(sample_count == 23){
							for(i = 0; i < 64 ; i++){
								ifft_re_out[i] = fft_im_in[i];
								ifft_im_out[i] = fft_re_in[i];
							}
						}

						//CPU to IFFT(fft) (real)
						else if(sample_count == 30){
							for(i = 0; i < 64; i++){
								ifft_re = ifft_re_out[i];
								XIo_Out32 ((XPAR_IFFT_RE_OUT_BASEADDR), ifft_re);
							}
						}

						//CPU to IFFT(fft) (image)
						else if(sample_count == 31){
							for(i = 0; i <64 ; i++){
								ifft_im = ifft_im_out[i];
								XIo_Out32 ((XPAR_IFFT_IM_OUT_BASEADDR), ifft_im);
							}
						}

						//input from ifft - ho operation Circuit
						Sample_AC97_out = XIo_In32(XPAR_HO_IN_BASEADDR);

	                    //output sample to AC97
						Sample_AC97_out = Sample_AC97_out<<2 ;
						XIo_Out32 ((AC97_BASEADDR + AC97_PCM_OUT_L_OFFSET), Sample_AC97_out);
						XIo_Out32 ((AC97_BASEADDR + AC97_PCM_OUT_R_OFFSET), Sample_AC97_out);

					}

					/*
					 *  first cycle (0 - 63 sample)
					 */
					else{
						flg_first_64_sample++;
					}

					if(sample_count == 32) sample_count = 0;
			}// PLAY Loop
			Xil_Out32(LED_BASEADDR, 0); //turn off LED

			//Set Tag and ID to configure the codec
			AC97_Set_Tag_And_Id (AC97_BASEADDR, 0xF800);
			//Mute
			AC97_Mute (AC97_BASEADDR, AC97_RECORD_GAIN_OFFSET);
    	}
    }

    cleanup_platform();

    return 0;
}
